@@ -15,6 +15,7 @@
 """Handle recipe parsing and substitution variables."""
 
 import datetime
+from functools import lru_cache
 import logging
 import os
 import subprocess
@@ -22,6 +23,7 @@ import sys
 from urllib.parse import urlparse
 
 import dateutil.parser
+from debian import debian_support
 
 
 MERGE_INSTRUCTION = "merge"
@@ -332,6 +334,14 @@ def fetch_branches(child_branch):
         "refs/heads/*:refs/remotes/%s/*" % child_branch.remote_name)
 
 
+@lru_cache(maxsize=1)
+def _git_version():
+    raw_git_version = subprocess.check_output(
+        ["dpkg-query", "-W", "-f", "${Version}", "git"],
+        universal_newlines=True)
+    return debian_support.Version(raw_git_version)
+
+
 def merge_branch(child_branch, target_path):
     """Merge the branch specified by `child_branch`.
 
@@ -343,9 +353,12 @@ def merge_branch(child_branch, target_path):
     fetch_branches(child_branch)
     try:
         child_branch.resolve_commit()
-        child_branch.git_call(
-            "merge", "--commit", "-m", "Merge %s" % child_branch.get_revspec(),
-            child_branch.commit)
+        cmd = ["merge", "--commit"]
+        if _git_version() >= "1:2.9.0":
+            cmd.append("--allow-unrelated-histories")
+        cmd.extend(["-m", "Merge %s" % child_branch.get_revspec()])
+        cmd.append(child_branch.commit)
+        child_branch.git_call(*cmd)
     except subprocess.CalledProcessError as e:
         raise MergeFailed(e.output)
 
